#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
import glob
import logging
import os
import shutil
from pathlib import Path
from typing import Dict, List

import requests

from .interfaces import ConveyorIF

"""
Post QA: Retrieve results directories from QA2 area to the spool area
"""


class RetrievalConveyor(ConveyorIF):
    def __init__(self, settings: Dict[str, str], action: str):
        self.logger = logging.getLogger("conveyor")
        self.action = action
        self.settings = settings

    def retrieval(self):
        qa_path = self.determine_qa_directory()
        spool_path = self.determine_spool_directory()

        os.chdir(spool_path)

        contents = glob.glob("*/", recursive=True)

        self.break_symlinks(spool_path, contents)
        if qa_path.exists():
            self.move_subdirectories_to_spool(qa_path, spool_path, contents)
            check = self.check_spool_contents(spool_path, contents)
            if check:
                Path(qa_path).rmdir()
        else:
            self.logger.info(f"Directories from {qa_path} have already been retrieved! Continuing...")

        if self.action == "QA Fail Cleanup":
            self.send_qa_fail_msg(self.settings["sdm_ids"])

    def send_qa_fail_msg(self, sdm_id: str):
        """
        Call REST endpoint that shoots off QA Fail message

        :return:
        """
        url = f"/workflows/requests/{self.settings['request_id']}/send-do-not-calibrate/{sdm_id}"
        requests.post(url, json={"sdm_id": sdm_id})

    def break_symlinks(self, spool_path: Path, dir_list: List[str]):
        self.logger.info(f"Breaking symlinks between qa2 and spool for directory {spool_path.stem}...")

        for directory in dir_list:
            directory = directory.strip("/")
            if Path(spool_path / directory).is_symlink():
                Path(spool_path / directory).unlink()
                self.logger.info(f'Symlink for "{directory}" removed.')
            else:
                self.logger.info(f'Directory "{directory}" is not a symlink! Skipping.')

    def move_subdirectories_to_spool(self, qa_path: Path, spool_path: Path, dir_list: List[str]):
        self.logger.info(f"Moving directories from qa2 to spool for directory {qa_path.stem}...")
        for directory in dir_list:
            qa_dir = qa_path / directory
            spool_dir = spool_path / directory
            self.logger.info(f"Moving directory {directory}...")
            shutil.move(qa_dir, spool_dir)
            self.logger.info("Done.")

    def determine_qa_directory(self) -> Path:
        if "Calibration" in self.action:
            qa_area = self.settings["qa_delivery_area"]
        else:
            qa_area = self.settings["image_qa_area"]

        wf_dir = self.settings["destination_subdir"]
        return Path(qa_area + "/" + wf_dir)

    def determine_spool_directory(self) -> Path:
        spool_area = self.settings["workspaces_lustre_root_dir"]
        wf_dir = self.settings["destination_subdir"]
        return Path(spool_area + "/" + wf_dir)

    def check_spool_contents(self, spool_path: Path, expected_dirs: List[str]) -> bool:
        os.chdir(spool_path)

        contents = glob.glob("*/", recursive=True)
        if set(contents).issubset(expected_dirs) and not len(contents) < len(expected_dirs):
            self.logger.info("All directories were successfully transferred!")
            return True
        else:
            self.logger.error(
                f"ERROR: A directory seems to be missing! "
                f"Directory list {contents} does not match the "
                f"list of expected directories!"
            )
            return False

    def convey(self):
        self.logger.info(f"RUNNING POST QA STANDARD {self.action.upper()}!")
        self.retrieval()
