#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
import argparse as ap
import datetime
import logging
import sys

import pandas as pd
from pyat import version
from pyat.schema.model import Filegroup
from pycapo import CapoConfig
from sqlalchemy import create_engine
from sqlalchemy.orm import (
    sessionmaker,  # an Engine, which the Session will use for connection
)

_NOW = datetime.datetime.utcnow()
_LOCATION = "DSOC"
_FILE_TYPE = "MARK4"
_MAX_ROWS_TO_IMPORT = 2

_LOG = logging.getLogger("mark4_import")
_LOG.setLevel(logging.INFO)
ch = logging.StreamHandler(sys.stdout)
formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
ch.setFormatter(formatter)
ch.setLevel(logging.DEBUG)
_LOG.addHandler(ch)
_DESCRIPTION = """Mark4 import version {}."""


def mark4_import():
    """
    Find matching idifits / mark4 pairs in the legacy archive. Create a mark4 file entry in the
    new archive and attach it to the same filegroup as the idifits file.
    :return:
    """
    args = _make_parser().parse_args()
    nconf = CapoConfig(profile=args.profile).settings("ngasDatabase")
    aconf = CapoConfig(profile=args.profile).settings("metadataDatabase")

    ngas_engine = create_engine(
        nconf.jdbcUrl.lstrip("jdbc:").replace("://", f"://{nconf.jdbcUsername}:{nconf.jdbcPassword}@")
    )
    archive_engine = create_engine(
        aconf.jdbcUrl.lstrip("jdbc:").replace("://", f"://{aconf.jdbcUsername}:{aconf.jdbcPassword}@")
    )

    ngasSession = sessionmaker(bind=ngas_engine)  # create a Session
    archiveSession = sessionmaker(bind=archive_engine)  # create a Session

    session = ngasSession()

    # Construct list of matching mark4 and idifits files in NGAS database
    mark4_matches = pd.read_sql(
        """
        select nf1.file_id file_path,
               nf1.file_id ngas_id,
               nf1.file_id filename,
               nf1.file_size filesize,
               nf2.file_id idifits
        from ngas_files nf1
        inner join ngas_files nf2
        on reverse(regexp_replace(reverse(nf1.file_name), '.*?_(.*)', '')) =
           reverse(regexp_replace(reverse(nf2.file_name), '.*?_(.*)', ''))
        where nf1.file_name like '%%mark4.tar.gz'
        and nf2.file_name like '%%.idifits'""",
        ngas_engine,
    )

    session.close()
    _LOG.info(f"{len(mark4_matches)} MARK4 files in NGAS with matching IDIFITS file")

    session = archiveSession()

    # Construct list of IDIFITS files in archive database with no matching MARK4 file
    idifits_matches = pd.read_sql(
        """
        select f1.filename idifits,
               f1.filegroup filegroup
        from files f1
        where f1.format = 'IDIFITS'
        and not exists (
            select 1
            from files f2
            where f2.format = 'MARK4'
            and reverse(regexp_replace(reverse(f2.filename), '.*?_(.*)', '')) =
                reverse(regexp_replace(reverse(f1.filename), '.*?_(.*)', ''))
            )
        """,
        archive_engine,
    )

    _LOG.info(f"{len(idifits_matches)} unmatched IDIFITS files in archive")

    file_merge = mark4_matches.merge(idifits_matches)
    _LOG.info(f"{len(file_merge)} merged entries")

    if len(file_merge) == 0:
        _LOG.info(f"No mark4 files found to be imported")
        session.close()
        exit(0)

    # Add in constant columns not in database
    file_merge["type"] = _FILE_TYPE
    file_merge["format"] = _FILE_TYPE
    file_merge["checksum"] = None
    file_merge["checksum_type"] = None
    file_merge["ingestion_time"] = _NOW
    file_merge["preview_storage_path"] = None
    file_merge["ngas_location"] = _LOCATION
    file_merge["ngas_cluster"] = _LOCATION

    # Prepare data for insert into database
    mark4_data = file_merge[
        [
            "file_path",
            "ngas_id",
            "filegroup",
            "filename",
            "filesize",
            "format",
            "type",
            "checksum",
            "checksum_type",
            "ingestion_time",
            "preview_storage_path",
            "ngas_location",
            "ngas_cluster",
        ]
    ]
    # Limit on number of rows to insert - for debugging
    # mark4_data = mark4_data.head(_MAX_ROWS_TO_IMPORT)

    # Append rows to file table in archive database
    mark4_data.to_sql("files", con=archive_engine, index=False, if_exists="append")

    # Update filesize in filegroup containing new mark4 files
    update_file_sizes(mark4_data, session)
    session.close()


def update_file_sizes(mark4_data, session):
    """
    Update file size for parent filegroup
    :param mark4_data: dataframe containing the merged data
    :param session:
    :return:
    """
    for row in range(len(mark4_data)):
        mark4_row = mark4_data.iloc[row]
        filegroup = session.query(Filegroup).filter(Filegroup.filegroup_id == int(mark4_row["filegroup"])).one_or_none()
        if filegroup is not None:
            filegroup.datasize = filegroup.datasize + int(mark4_row["filesize"])
            session.add(filegroup)
        else:
            _LOG.warning(f"Filegroup not found for id {mark4_row['filegroup']}")
    session.commit()


def _make_parser():
    r"""Build a command line parser for this app: this is external to MrBooks and/or CapoSettings because both need it,
    and Sphinx will want a look at ot to build docs."""
    result = ap.ArgumentParser(description=_DESCRIPTION.format(version), formatter_class=ap.RawTextHelpFormatter)
    result.add_argument("-P", "--profile", action="store", help="profile name to use, e.g. test, production")
    return result


if __name__ == "__main__":
    mark4_import()
