#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
""" Objects pertaining to the various ingestion manifests """

from __future__ import annotations

from enum import Enum

# pylint: disable=E0401, R0903, R1721, W0622
from pathlib import Path
from typing import List


class Telescope(Enum):
    """Codifying names of telescopes that appear in ingestion manifests"""

    VLA = "VLA"
    EVLA = "EVLA"
    ALMA = "ALMA"
    VLBA = "VLBA"
    GBT = "GBT"
    NONE = "NONE"
    UNKNOWN = "UNKNOWN"


class ScienceProductType(Enum):
    """Canonical collection of ingestible types of science products"""

    EXEC_BLOCK = "execution_block"
    EVLA_CAL = "calibration"
    CATALOG = "catalog"
    IMAGE = "image"


class AncillaryProductType(Enum):
    """The various types of ancillary products we'll encounter"""

    INGESTION_ARTIFACTS = "ingestion_artifacts"
    PIPELINE_ARTIFACTS = "pipeline_artifacts"
    PIPELINE_WEBLOG_TYPE = "pipeline_weblog"
    LOG_TYPE = "log_file"

    ### Images ###

    # our default FITS type
    FITS = "fits_image"

    QUICKLOOK_IMAGE = "quicklook_image"
    QUICKLOOK_RMS_IMAGE = "quicklook_rms_image"
    FITS_MASK = "clean_mask"
    PB_FITS = "primary_beam"
    ALPHA_FITS = "spectral_index"
    CANDIDATE_IMG = "candidate_image"
    THUMBNAIL_IMG = "thumbnail_image"


class InvalidLocatorException(Exception):
    """Throw this if we're fed a bad science product locator"""


class IngestionManifestException(Exception):
    """Throw this if we're unable to construct an ingestion manifest using supplied inputs"""


def find_output_tars(files_found, staging_source_dir) -> List[Path]:
    """
    Round up the output science products associated with this SP type.

    :return:
    """
    tar_files = [file for file in files_found if file.name.endswith(".tar")]

    if len(tar_files) == 0:
        raise IngestionManifestException(f"No output science products found at {staging_source_dir}")

    return tar_files
